import warnings
warnings.filterwarnings("ignore")

from .preprocess import *
from .data_loading import *
from .basicNetwork import *
from .string_manipulate import *
from .opt_gaussian import *
from .data_stats import *
from .IO import *
from .dependency import *
from .DManager import *
from .imputation import *
from .type_check import *
from .kde import *
from .L2X import *
from .gumbel import *
from .run_k_fold_on_function import *
from .sampling import *
from .operations import *
from .matrix_manipulate import *
from .explainer import *
from .dimension_reduction import *
from .system import *
from .regression import *
from .classification import *
from .cluster import *
from .kernels import *
from .IKDR import *
from .flow import flow
from .generate_data import *
from .feature_selection import *
from .distributions import *
from .dates_and_time import *
from .hyperparameter_tuning import *
from .rebalance_data import *
from .random_features import *
from .distance_between_distributions import *



#!/usr/bin/env python

from wplotlib import histograms
import numpy as np
import wuml
import torch


#str.ljust(s, width[, fillchar])
#str.rjust(s, width[, fillchar])
#str.center(s, width[, fillchar])


def pretty_np_array(m, front_tab='', verticalize=False, title=None, auto_print=False, end_space=''):
	m = str(m)

	if type(m) == type(torch.tensor([])):
		m = m.cpu().detach().numpy()

	if verticalize:
		if len(m.shape) == 1:
			m = np.atleast_2d(m).T

	out_str = front_tab + str(m).replace('\n ','\n' + front_tab).replace('[[','[').replace(']]',']') + end_space + '\n'
	out_str = str(out_str).replace('.]',']')

	if type(title).__name__ == 'str':
		L1 = out_str.split('\n')
		L1_max_width = len(max(L1, key=len))
		t1 = str.center(title, L1_max_width)
		out_str = t1 + '\n' + out_str

	if auto_print: print(out_str)
	else: return out_str

def block_two_string_concatenate(str1, str2, spacing='\t', add_titles=[], auto_print=False):
	str1 = str(str1)
	str2 = str(str2)

	L1 = str1.split('\n')
	L2 = str2.strip().split('\n')

	if len(L1) > len(L2):
		Δ = len(L1) - len(L2)
		for ι in range(Δ):
			L2.append('\n')

	if len(add_titles) == 2:
		L1_max_width = len(max(L1, key=len))
		L2_max_width = len(max(L2, key=len))
		t1 = str.center(add_titles[0], L1_max_width)
		t2 = str.center(add_titles[1], L2_max_width)
		L1.insert(0,t1)
		L2.insert(0,t2)

	max_width = len(max(L1, key=len))
	outS = ''
	for l1, l2 in zip(L1,L2):
		outS += ('%-' + str(max_width) + 's' + spacing + l2 + '\n') % l1

	if auto_print: print(outS)
	else: return outS

def print_two_matrices_side_by_side(M1, M2, title1='', title2='', auto_print=True):
	eK = pretty_np_array(M1, front_tab='', title=title1, auto_print=False)
	eQ = pretty_np_array(M2, front_tab='', title=title2, auto_print=False)
	block_two_string_concatenate(eK, eQ, spacing='\t', add_titles=[], auto_print=auto_print)

def block_string_concatenate(strList, spacing='\t'):
	all_concat = ''
	for single_block in strList:

		if all_concat == '':
			all_concat = single_block
		else:
			all_concat = block_two_string_concatenate(all_concat, single_block, spacing=spacing)

	return all_concat

if __name__ == "__main__":
	str1 = 'Hello world\nA interesting string'
	str2 = 'Second Block\nto the right\nTooLong'
	outS = block_two_string_concatenate(str1, str2)
	print(outS)

	str1 = 'Hello world\nA interesting string'
	str2 = 'Second Block, to the right, TooLong'
	str3 = 'Third Block, \n make it shorter'
	all_3 = [str1, str2, str3]
	outS = block_string_concatenate(all_3)
	print(outS)






import sklearn
import os
import sys
if os.path.exists('/home/chieh/code/wuML'):
	sys.path.insert(0,'/home/chieh/code/wuML')

import wuml
from sklearn.datasets import make_classification

def make_moons(n_samples=100, shuffle=True, noise=0.05, random_state=None):
	(X,Y) = sklearn.datasets.make_moons(n_samples=n_samples, shuffle=shuffle, noise=noise, random_state=random_state)

	data = wuml.ensure_wData(X, column_names=None)
	data.Y = Y
	return data

def make_classification_data( n_samples=100, n_features=20, n_informative=2, n_redundant=0, n_repeated=0, 
								n_classes=2, n_clusters_per_class=2, weights=None, flip_y=0.01, class_sep=1.0, 
								hypercube=True, shift=0.0, scale=1.0, shuffle=True, random_state=None, encode_discrete_label_to_one_hot=False):

	X, y = make_classification(n_samples=n_samples, n_features=n_features, n_informative=n_informative, n_redundant=n_redundant, n_repeated=n_repeated, 
								n_classes=n_classes, n_clusters_per_class=n_clusters_per_class, weights=weights, flip_y=flip_y, class_sep=class_sep, 
								hypercube=hypercube, shift=shift, scale=scale, shuffle=shuffle, random_state=random_state)


	data = wuml.wData(X_npArray = X, Y_npArray=y, label_type='discrete', encode_discrete_label_to_one_hot=encode_discrete_label_to_one_hot)
	return data





#!/usr/bin/env python
import os 
import sys

if os.path.exists('/home/chieh/code/wPlotLib'):
	sys.path.insert(0,'/home/chieh/code/wPlotLib')
if os.path.exists('/home/chieh/code/wuML'):
	sys.path.insert(0,'/home/chieh/code/wuML')

import wuml
import wplotlib
from wuml import jupyter_print
from ot.datasets import make_1D_gauss as gauss
import numpy as np
import ot



#	X and Y can be a matrix or a vector
#	if they are vectors, we assume they are counts of the histogram, returning a scalar value for the distance
#	if they are matrices, we assume each row is a histogram and output a distance matrix
#	if M is none, we assume the increments of distance is 1 between tickers
def wasserstein_distance(X, Y, M=None, XY_as_histogram=True, output_as='ndarray', method='set negative to small noise'):

	X = np.squeeze(wuml.ensure_numpy(X))
	Y = np.squeeze(wuml.ensure_numpy(Y))

	if XY_as_histogram:
		if len(X.shape) == 1:
			Nx = X.shape[0]
			if M is None:
				D = np.arange(Nx, dtype=np.float64)
				D = D.reshape((Nx, 1))
				M = ot.dist(D, D, 'euclidean')

			# normalize the distribution to 1
			X = wuml.map_vector_to_distribution_data(X, method=method)
			Y = wuml.map_vector_to_distribution_data(Y, method=method)
			
			return ot.emd2(X, Y, M)

		elif len(X.shape) == 2:
			n = X.shape[0]
			wasDis_matrix = np.zeros((n,n))
			for i in range(n):
				for j in range(n):
					Nx = X[i].shape[0]
					if M is None:
						D = np.arange(Nx, dtype=np.float64)
						D = D.reshape((Nx, 1))
						M = ot.dist(D, D, 'euclidean')

					if X[i].flags['C_CONTIGUOUS'] == False: Xc = X[i].copy(order = 'C')
					else: Xc = X[i]
					if Y[j].flags['C_CONTIGUOUS'] == False: Yc = Y[j].copy(order = 'C')
					else: Yc = Y[j]

					# normalize the distribution to 1
					Xc = wuml.map_vector_to_distribution_data(Xc, method=method)
					Yc = wuml.map_vector_to_distribution_data(Yc, method=method)
					wasDis_matrix[i,j] = ot.emd2(Xc, Yc, M)

					wuml.write_to_current_line('Completion %.3f, at (%d,%d) -> %.4f'%((i*n+j)/(n*n), i, j, wasDis_matrix[i,j]))
			print('\n')
			if output_as == 'ndarray': return wasDis_matrix
			elif output_as == 'wData': return wuml.wData(X_npArray=wasDis_matrix) # exact linear program
		else:
			raise ValueError('Wasserstein Distance does not handle 3 dimensional data.')
	else:
		raise ValueError('XY_as_histogram as False is not yet supported')







import sklearn.metrics
import numpy as np
import wuml

def rbk_kernel(X, σ, zero_diagonal=False, use_random_features=False, random_feature_method='rff', random_feature_width=500):
	#random_feature_method = 'rff' or 'orthogonal'

	if use_random_features:
		sorf = wuml.random_feature(sigma=σ, random_feature_method=random_feature_method, sample_num=random_feature_width)
		Kx = sorf.get_kernel(X)			# kernel matrix from orthogonal
		if zero_diagonal: np.fill_diagonal(Kx, 0)			#	Set diagonal of adjacency matrix to 0
	else:
		γ = 1.0/(2*σ*σ)
		Kx = sklearn.metrics.pairwise.rbf_kernel(X, gamma=γ)
		if zero_diagonal: np.fill_diagonal(Kx, 0)			#	Set diagonal of adjacency matrix to 0

	return Kx

def get_rbf_γ(X):
	σ = np.median(sklearn.metrics.pairwise.pairwise_distances(X))
	γ = 1.0/(2*σ*σ)
	return γ




#!/usr/bin/env python

from collections import Counter
from sklearn.datasets import make_classification
from imblearn.over_sampling import SMOTE 
import wuml
import numpy as np


class rebalance_data:
	def __init__(self, data, y=None, method='smote'):

		label_column_name=None
		if y is None:
			if wuml.wtype(data) == 'wData':
				y = X.Y
				X = data.X
			else:
				raise ValueError('data must be wData if the label y is None')
			label_column_name = data.get_column_names_as_a_list()
		else:
			X = wuml.ensure_numpy(data)
			y = wuml.ensure_numpy(y)
			
	
		sm = SMOTE(random_state=42)
		X_res, y_res = sm.fit_resample(X, y)
	
		self.balanced_data = wuml.wData(X_npArray=X_res, Y_npArray=y_res, first_row_is_label=False, label_column_name=label_column_name, label_type='discrete')

if __name__ == "__main__":
	X = np.random.randn(100, 10)
	y = np.append(np.ones(20), np.zeros(80))

	rebalancer = rebalance_data(X, y)
	print(rebalancer.balanced_data.shape)
	print(rebalancer.balanced_data)
	import pdb; pdb.set_trace()
